<%Page path="/hints/bits/header"%>

<%Styler text="Content -- Instruction" style="hint-title"%>

Instruction is displayed on the content editing page. It is supposed to
describe all limitations and suggestions for the person editing that
content.
<P>

Instruction is displayed as is and should contain valid HTML code.
<P>

<%Page path="/hints/bits/footer"%>
