<%Page path="/admin/bits/header" PAGENAME="content"%>

<%Styler text="Content Editing" style="page-title"%>

<%Condition
  a.cgiparam="store"
  a.path="/bits/content/edit-store"
  default.path="/bits/content/edit-show"
%>

<%Page path="/admin/bits/footer"%>
