<%Page path="/hints/bits/header"%>

<%Styler text="Content -- Name Syntax" style="hint-title"%>

Content name uniquely refers to a dynamic content element throughout the
site.
<P>

Allowed characters are letters of english alphabet, digits and
underscore sign. Maximum length is limited to 30 characters.
<P>

Usually names are grouped by areas with are name first, then underscore
and some name inside of that group.
<P>

Examples:
<PRE>
  about_management
  about_staff
  contact_ca
  contact_nj
</PRE>
<P>

<%Page path="/hints/bits/footer"%>
