<%Page path="/hints/bits/header"%>

<%Styler text="Content -- Preview URL" style="hint-title"%>

Preview URL should have the URL of the page that best demonstrates the
dynamic content you create.
<P>

For instance, if you create content for "About US" section then you
should use the URL of the "About US" page on the site.
<P>

If there is no distinct page that can demonstrate the content you can
leave the "Preview URL" field blank. But whenever possible it is better
to use correct Preview URL.
<P>

<%Page path="/hints/bits/footer"%>
