<%Page path="/hints/bits/header"%>

<%Styler text="Content -- Description" style="hint-title"%>

Description field provides a space for more details about the
content. It does not have any limitations for the characters used, but
should be limited to 100 characters in length.
<P>

<%Page path="/hints/bits/footer"%>
